package org.ids_mannheim;

import org.apache.commons.compress.compressors.CompressorException;
import org.apache.commons.compress.compressors.CompressorInputStream;
import org.apache.commons.compress.compressors.CompressorStreamFactory;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.io.TempDir;

import static org.junit.jupiter.api.Assertions.*;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Logger;

public class PseudonymizeTest {
    private static final ByteArrayOutputStream outContent = new ByteArrayOutputStream();
    private static final ByteArrayOutputStream errContent = new ByteArrayOutputStream();
    private static final PrintStream originalOut = System.out;
    private static final PrintStream originalErr = System.err;

    @BeforeAll
    public static void setUpStreams() {
        System.setOut(new PrintStream(outContent));
        System.setErr(new PrintStream(errContent));
    }

    @AfterAll
    public static void restoreStreams() {
        System.setOut(originalOut);
        System.setErr(originalErr);
    }

    @BeforeAll
    public static void disableGrapeGrab() {
        System.setProperty("groovy.grape.enable", "false");
    }

    @Test
    void generateKeyTest() {
        GeneratePseudonymKey.main(new String[]{"src/test/resources/token_lemma_pos_freqs.tsv.xz"});
        assertTrue(outContent.toString().contains("\ndurchaus\t934\n"));
        assertTrue(outContent.toString().contains("\nlängst\t936\n"));
        assertFalse(outContent.toString().contains("\t937\n"));
    }

    @Test
    void cardinalsArePseudonymizedCorrectlyTest() {
        GeneratePseudonymKey.main(new String[]{"src/test/resources/cardinals_test_lemma_pos_freqs.tsv.gz"});
        assertTrue(outContent.toString().contains("\n1000\t1"));
    }

    @Test
    void filterTest() throws IOException, CompressorException {
        final String TOKEN_KEY = "cardinals_token_key.tsv.gz";
        final String LEMMA_KEY = "cardinals_lemma_key.tsv.gz";

        Path tmpDir = Files.createTempDirectory("totalngrams");
        String tmpDirPath = Files.createTempDirectory("totalngrams").toFile().getAbsolutePath();
        Files.copy(Path.of("src/test/resources/"+ TOKEN_KEY), Paths.get(tmpDirPath, TOKEN_KEY));
        Files.copy(Path.of("src/test/resources/"+ LEMMA_KEY), Paths.get(tmpDirPath, LEMMA_KEY));
        FilterKeys.main(new String[]{
                "-k", String.valueOf(Paths.get(tmpDirPath, TOKEN_KEY)),
                "-k", String.valueOf(Paths.get(tmpDirPath, LEMMA_KEY)),
                "src/test/resources/pseudo_cardinals_test_lemma_pos_freqs.0.tsv.gz"
        });
        CompressorInputStream inputStream = new CompressorStreamFactory().createCompressorInputStream(new BufferedInputStream(new FileInputStream(Paths.get(tmpDirPath, "cardinals_token_key.0.tsv.gz").toFile())));
        String filteredKeys = new String(inputStream.readAllBytes(), StandardCharsets.UTF_8);
        assertEquals (
                "123\t0\n" +
                "1000\t1\n" +
                "Anfang\t2\n" +
                "Ende\t3\n", filteredKeys);

        inputStream = new CompressorStreamFactory().createCompressorInputStream(new BufferedInputStream(new FileInputStream(Paths.get(tmpDirPath, "cardinals_lemma_key.0.tsv.gz").toFile())));
        filteredKeys = new String(inputStream.readAllBytes(), StandardCharsets.UTF_8);
        assertEquals (
                "123\t0\n" +
                        "1000\t1\n" +
                        "Anfang\t2\n" +
                        "Ende\t3\n", filteredKeys);
        Files.delete(tmpDir);
    }

    @Test
    void readKeyTest() {
        ConcurrentHashMap map = (ConcurrentHashMap) Pseudonymize.readKey("src/test/resources/token.keys.gz", Logger.getLogger(""));
        assertEquals(936, map.get("längst"));
    }

    @TempDir
    static Path sharedTempDir;

    @Test
    void pseudonymizeFilesTest() {
        Logger log = Logger.getLogger("");
        String fname = sharedTempDir + File.separator + "token_lemma_pos_freqs.tsv.xz";
        new File(fname).delete();
        Pseudonymize.pseudonymizeFiles(new String[]{"src/test/resources/token_lemma_pos_freqs.tsv.xz"},
                new String[]{"src/test/resources/token.keys.gz", "src/test/resources/lemma.keys.gz"}, sharedTempDir.toAbsolutePath().toString(), log);
        ConcurrentHashMap pseudonymized = (ConcurrentHashMap) Pseudonymize.readKey(fname, log);
        assertEquals(223997, pseudonymized.get("936"));
    }
}